<?php
/**
 * WP Malware Removal
 *
 * @package     WP Malware Removal
 * @author      Shivanand Sharma
 * @copyright   2019 wp-malware-removal.com
 * @license     MIT
 *
 * @wordpress-plugin
 * Plugin Name: WP Malware Removal
 * Description: Ultra-precision, comphrensive malware scanner (and file inspector) for your site to find viruses, infections & other security threats & vulnerabilities.
 * Version:     2.3
 * Author:      Shivanand Sharma
 * Author URI:  https://wp-malware-removal.com
 * Text Domain: wpmr
 * License:     MIT
 * License URI: https://opensource.org/licenses/MIT
 * Plugin URI:  https://wp-malware-removal.com
 */

/*

██╗    ██╗██████╗     ███╗   ███╗ █████╗ ██╗     ██╗    ██╗ █████╗ ██████╗ ███████╗    ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗ █████╗ ██╗     
██║    ██║██╔══██╗    ████╗ ████║██╔══██╗██║     ██║    ██║██╔══██╗██╔══██╗██╔════╝    ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔══██╗██║     
██║ █╗ ██║██████╔╝    ██╔████╔██║███████║██║     ██║ █╗ ██║███████║██████╔╝█████╗      ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║███████║██║     
██║███╗██║██╔═══╝     ██║╚██╔╝██║██╔══██║██║     ██║███╗██║██╔══██║██╔══██╗██╔══╝      ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══██║██║     
╚███╔███╔╝██║         ██║ ╚═╝ ██║██║  ██║███████╗╚███╔███╔╝██║  ██║██║  ██║███████╗    ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ██║  ██║███████╗
 ╚══╝╚══╝ ╚═╝         ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝ ╚══╝╚══╝ ╚═╝  ╚═╝╚═╝  ╚═╝╚══════╝    ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚═╝  ╚═╝╚══════╝
                                                                                                                                                    
          ██████╗ ██╗   ██╗    ███████╗██╗  ██╗██╗██╗   ██╗                                                                                         
          ██╔══██╗╚██╗ ██╔╝    ██╔════╝██║  ██║██║██║   ██║                                                                                         
█████╗    ██████╔╝ ╚████╔╝     ███████╗███████║██║██║   ██║                                                                                         
╚════╝    ██╔══██╗  ╚██╔╝      ╚════██║██╔══██║██║╚██╗ ██╔╝                                                                                         
          ██████╔╝   ██║       ███████║██║  ██║██║ ╚████╔╝                                                                                          
          ╚═════╝    ╚═╝       ╚══════╝╚═╝  ╚═╝╚═╝  ╚═══╝                                                                                           
                                                                                                                                                    

*/


define( 'WPMR_PLUGIN_DIR_URL', trailingslashit(plugin_dir_url( __FILE__ )));
define( 'WPMR_SERVER','https://wp-malware-removal.com/' );
define( 'WPMR_PLUGIN', __FILE__ );
define( 'WPMR_PLUGIN_DIR', __DIR__ );

final class WPMR_Init {

    private $definitions;
    private $flag;
    public $dir ;
    public $url ;

    static function get_instance() {
        static $instance = null;
            if ( is_null( $instance ) ) {
                $instance = new self;
                $instance->init();
            }
            return $instance;
    }

    function init(){
        
        $this->dir = trailingslashit( plugin_dir_path( __FILE__ ) );
        $this->url = trailingslashit( plugin_dir_url(  __FILE__ ) );

        add_action( 'upgrader_process_complete', array($this, 'delete_checksums'), 10, 2);

        add_action( 'admin_init', array( $this, 'set_plugin_data') );
        add_action( 'admin_init', array($this, 'save_gsc_profile' ));

        add_filter( 'plugin_action_links_' . plugin_basename( WPMR_PLUGIN ), array( $this, 'plugin_action_links' ) );
        add_filter( 'plugin_row_meta', array($this, 'plugin_meta_links'), 10, 2 ); // Add important links to plugins list (right-side)

        add_action( 'admin_enqueue_scripts', array( $this, 'plugin_styles' ) );
        add_action( 'admin_menu', array( $this, 'settings_menu' ) );

        
        add_action( 'wp_ajax_wpmr_init_scan', array( $this, 'wpmr_init_scan' ));
        add_action( 'wp_ajax_wpmr_scan_files', array( $this, 'wpmr_scan_files' ));
        add_action( 'wp_ajax_wpmr_inspect_file', array( $this, 'wpmr_inspect_file' ));
        
        add_action( 'wp_ajax_wpmr_update_sigs', array( $this, 'update_definitions' ));
        add_action( 'wp_ajax_wpmr_register', array( $this, 'wpmr_register' ));
        
        add_action( 'wp_ajax_nopriv_wpmr_inspect_file', '__return_false');
        add_action( 'wp_ajax_nopriv_wpmr_init_scan', '__return_false' );
        add_action( 'wp_ajax_nopriv_wpmr_scan_files', '__return_false' );
        add_action( 'wp_ajax_nopriv_wpmr_check_files', '__return_false' );
        add_action( 'wp_ajax_nopriv_wpmr_update_sigs', '__return_false' );
        add_action( 'wp_ajax_nopriv_wpmr_register', '__return_false' );
        
        add_action( 'admin_footer',  array( $this, 'scripts'));

        
        add_action( "load-toplevel_page_wpmr", array($this, 'load_meta_boxes' ));
        add_action( 'load-toplevel_page_wpmr', array($this, 'add_meta_boxes' ));
        add_action( 'load-toplevel_page_wpmr', array( $this, 'add_admin_scripts' ) );
        add_action( 'admin_head', array( $this, 'admin_style' ) );
        add_action( 'in_plugin_update_message-'. basename(WPMR_PLUGIN_DIR).'/'.basename(WPMR_PLUGIN), array($this, 'plugin_update_message'), 10, 2 );

    }

    private function __construct() {
    }

    function load_meta_boxes(){
        add_action( 'add_meta_boxes', array($this, 'wpmr_add_metaboxes' ));
    }

    function wpmr_add_metaboxes(){
        add_meta_box( 'wpmr-results-box', 'Scan Results', array($this, 'meta_box_results'), 'toplevel_page_wpmr', 'main', 'high' );
        add_meta_box( 'wpmr-inspect-box', 'File Inspector', array($this, 'meta_box_inspect'), 'toplevel_page_wpmr', 'main', 'high' );
        add_meta_box( 'wpmr-updates-box', 'Updates', array($this, 'meta_box_updates'), 'toplevel_page_wpmr', 'side', 'high' );
        add_meta_box( 'wpmr-support-box', 'About WP Malware Removal', array($this, 'meta_box_support'), 'toplevel_page_wpmr', 'side', 'high' );
        add_meta_box( 'wpmr-tools-box', 'External Tools', array($this, 'meta_box_wpmrtools'), 'toplevel_page_wpmr', 'side', 'high' );
    }

    function add_meta_boxes(){
        do_action( 'add_meta_boxes', 'toplevel_page_wpmr' );
    }

    function meta_box_updates() { ?>
        <div class="wpmr-updates">
            <table id="server">
                <tr>
                    <th>PHP:</th><td><?php echo phpversion(); ?></td>
                </tr>
                <tr>
                    <th>Server:</th><td><?php echo $_SERVER['SERVER_SOFTWARE']; ?></td>
                </tr>
                <tr>
                    <th>WordPress:</th><td><?php global $wp_version; echo $wp_version; ?></td>
                </tr>
                <tr>
                    <th>Definitions:</th><td><?php echo get_transient('WPMR_signatures')['v']  ; ?></td>
                </tr>
                <tr>
                    <th>Last Updated:</th><td id="last_updated">
                    <?php 
                    echo $this->get_last_updated_ago();
                    ?>
                    </td>
                </tr>
            </table>
            <?php
            if( $this->is_registered() ) {
                $reg = get_option('WPMR_reg');
                $statevars = $this->encode(array_merge($this->get_diag_data(), $reg ));
                echo '<div id="is_registered">';
                if(empty($reg['gsc'])){
                    echo '<p style="border-top: 2px solid #ccc; padding: 1em 0"><strong>Connection to Google Search Console helps to</strong></p>
                    <ol>
                    <li>Detect and report security issues in time.</li>
                    <li>Automatically notify you if your SERPs results are hijacked.</li>
                    </ol>
                    <p><strong>It just takes less than a minute to connect.</strong></p>';
                    echo '<a class="button-primary" href="'. WPMR_SERVER . '?p=495&wpmr_authenticate=' . $statevars  .'">Authenticate with Google Search Console</a>';
                }
                echo '<h3>Force Update Signatures</h3>';
                submit_button( 'Update Signatures', 'primary', 'wpmr-update' );
                echo '</div>';
            }
            else {
                echo '<div id="is_unregistered"><h3 style="border-top: 2px solid #ccc; padding: 1em 0; line-height: 1.618em"><strong>Super!<br />Let\'s get this thing to work.<br />Click on the button below to receive updated definitions. Registering helps you…</strong></h3>
                <ol>
                <li>Detect and report security issues in time.</li>
                <li>Auto-update the definitions.</li>
                <li>You get quality service.</li>
                <li>My server doesn\'t go down due to spam-bots.</li>
                </ol>
                <p><strong>This is totally free.</strong></p>';
                $current_user = wp_get_current_user();
                ?>
                <form id="wpmp_reg_form">
                <table id="wpmr_reg">
                <tr><td>First name</td><td><input type="text" name="wpmr_fn" id="wpmr_fn" required value="<?php echo $current_user->user_firstname ?>" /></td></tr>
                    <tr><td>Last name</td><td><input type="text" name="wpmr_ln" id="wpmr_ln" required value="<?php echo $current_user->user_lastname ?>" /></td></tr>
                    <tr><td>Email</td><td><input type="email" name="wpmr_eml" id="wpmr_eml" required value="<?php echo $current_user->user_email ?>" /></td></tr>
                    <tr><td>Site</td><td><input type="URL" readonly name="wpmr_url" id="wpmr_url" required value="<?php echo site_url(); ?>" /></td></tr>
                    <tr><td>Key</td><td><input type="text" readonly name="wpmr_key" id="wpmr_key" required value="<?php echo md5(site_url()); ?>" /></td></tr>
                </table>
                </form>
            <script type="text/javascript">
            jQuery(document).ready(function ($) { //wrapper
                $("#wpmr-register").click(function (e) {
                    e.preventDefault();
                    if(! document.querySelector('#wpmp_reg_form').reportValidity()){
                        return false;
                    }
                    wpmr_register = {
                        wpmr_register_nonce: '<?php echo wp_create_nonce( 'wpmr_register' ); ?>',
                        action: "wpmr_register",
                        cachebust: Date.now(),
                        user: {
                            fn: $('#wpmr_fn').val(),
                            ln: $('#wpmr_ln').val(),
                            email: $('#wpmr_eml').val(),
                            key: $('#wpmr_key').val()
                        }
                    };
                    
                    $.ajax({
                        url: ajaxurl,
                        method: 'POST',
                        data: wpmr_register,
                        complete: function(jqXHR, textStatus){
                        },
                        success: function (response) {
                            console.dir( response );
                            if( (typeof response ) != 'object' ) {
                                response = JSON.parse(response);
                            }
                            if(response.hasOwnProperty('success')) {
                                $('#is_unregistered').html('<p><strong>Thank you for registering! Definition auto-update enabled.</strong></p>');
                                location.reload(true);
                            }
                            else {
                                $('#reg_error').html('<p><strong>' + response.error + '</strong></p>');
                            }
                        } // initialize
                    }); // ajax post
                    return false;
                });
            });
            </script>
                <?php
                echo '<div id="reg_error"></div>';
                submit_button( 'Free Registration', 'primary', 'wpmr-register' );
                echo '</div>';
            }
            ?>
        </div>
        <?php
    }

    function meta_box_wpmrtools() {
        $domain = untrailingslashit(preg_replace('/https?:\/\//','',get_site_url()));
        $domain = explode('/',$domain)[0];
        echo '
        <p><strong>Click on any of the links below to run your site through the tool.</strong> All links open in new tab / window.</p>
        <p><strong>Note:</strong>This tool is not affiliated with any of these sites.</p>
        <ol>
        <li><a id="tool" target="_blank" href="https://www.google.com/transparencyreport/safebrowsing/diagnostic/index.html#url='.get_site_url().'">Google Safe Browsing Diagnostic</a></li>
        <li><a id="tool" target="_blank" href="https://sitecheck.sucuri.net/results/'.get_site_url().'">Sucuri Website Malware Scanner</a></li>
        <li><a id="tool" target="_blank" href="https://www.virustotal.com/#/domain/'. $domain . '">VirusTotal Analyzer</a><br></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/screenshot/'.get_site_url().'">Geek Flare Screenshot Checker</a></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/blacklist-test/'.get_site_url().'">Geek Flare Blacklist Test</a></li>
        <li><a id="tool" target="_blank" href="https://geopeeker.com/fetch/?url='.get_site_url().'">GeoPeeker Peek Results</a><br></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/dnssec-test/'.get_site_url().'">Geek Flare DNSSEC Test</a></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/https-test/'.get_site_url().'">Geek Flare HTTP/2 Test</a></li>
        </ol>
        ';
    }

    function meta_box_results(){ ?>
        <div class="scan-results">
        <div id="scan-details"></div>
        <div id="scan-results"></div>
            <div id="wpmr-what-to-do">                
                <p><strong>LEGEND:</strong> <span class="skipped legend threat" title="This file is probably a threat">Error reading file</span> <span class="probable legend threat" title="This file is probably a threat">Probably Infected</span> <span class="severe legend threat" title="This file is a severe threat">Severely Infected</span></p>
                <h1>What to do in case of malware infection:</h1>
                <ol>
                    <li>
                        <strong>Please do manual investigation before deleting any files blindly.</strong>
                    </li>
                    <li>
                        <strong>In case of any possible infection, put your site into maintenance mode, take full backup and <a href="https://wp-malware-removal.com/?p=44" target="_blank">file malware clean-up request</a>.</strong>
                    </li>
                </ol>
                <p><strong><a href="https://wp-malware-removal.com/?p=44" target="_blank" class="button-primary">File malware clean-up request</a></strong></p>
                <p><strong><a href="https://twitter.com/shivtwin" target="_blank" class="button-secondary">Connect on Twitter</a></strong></p>
            </div>
        </div>
        <?php
    }
    
    function meta_box_inspect(){
        ?>
        <span id="inspect_file_path"></span>
        <pre id="inspect_file"></pre>
        <a href="#wpmr-results-box" class="button-primary wpmr-back">Go Back to Results</a>
        <?php
    }

    function meta_box_support(){ ?>
            <div class="scan-support">
            <p><img src="<?php echo $this->url; ?>assets/shiv8288-72.jpg"></p>
            <p>Hi, I’m Shiv — the author of WP Malware Removal. I work in defense as an IT information-security analyst with a career spanning over 18+ years.</p>
            <p><strong>WP Malware Removal serves 40,000+ definition updates everyday.</strong> Your donations support further development of this plugin. <strong><a target="_blank" href="https://www.converticacommerce.com?item_name=Donation%20for%20WP%20Malware%20Removal&cmd=_donations&currency_code=USD&lc=US&business=shivanand@converticacommerce.com">Please consider a donation.</a></strong></p>
            <p>If you find a bug in the plugin, do not hesitate to contact me.</p>
                <p><strong><a href="https://wp-malware-removal.com/?p=300" target="_blank" class="button-primary">File support request</a></strong></p>
                <p><strong><a href="https://twitter.com/shivtwin" target="_blank" class="button-secondary">Twitter</a></strong></p>
            </div>
        <?php
    }

    function save_gsc_profile(){
        if ( isset( $_REQUEST['wpmr-action'] ) && $_REQUEST['wpmr-action'] == 'oauth' ) {
            wp_verify_nonce( $_REQUEST['origin_nonce'], 'wpmr_gscapi' );
            if( current_user_can('activate_plugins') 
            && isset($_REQUEST['success']) && $_REQUEST['success']
            ){
                $wpmr_reg = get_option('WPMR_reg');
                if($wpmr_reg ) {
                    $wpmr_reg['gsc'] = true;
                    update_option('WPMR_reg', $wpmr_reg);
                }
            }
            else {  
            }
            wp_redirect( esc_url( get_admin_url( null, 'options-general.php?page=wpmr') ), 302 );
            exit;
        }
        
        if ( isset( $_REQUEST['wpmr-action'] ) && $_REQUEST['wpmr-action'] == 'revoke' && isset( $_REQUEST['success'] ) && $_REQUEST['success'] == '1' ) {
            $wpmr_reg = get_option('WPMR_reg');
            if($wpmr_reg && !empty($wpmr_reg['gsc'])) {
                unset($wpmr_reg['gsc']);
                update_option('WPMR_reg', $wpmr_reg);
            }
        }
    }

    function set_plugin_data(){
        if(is_admin()){
            $this->plugin_data = get_plugin_data( WPMR_PLUGIN, false, false );
        }
    }

    function admin_style(){ ?>
        <style type="text/css">#toplevel_page_wpmr .wp-menu-image img { width: 30px; height: auto; opacity: 1; padding: 0 0 0 0;}</style>
        <?php
    }

    function wpmr_register(){
        check_ajax_referer( 'wpmr_register', 'wpmr_register_nonce' );

        global $wp_version;
        
        if( empty($_REQUEST['user']) || empty($_REQUEST['user']['email']) || empty($_REQUEST['user']['fn']) || empty($_REQUEST['user']['ln']) ) {
            wp_send_json( array('error' => 'Please fill all details') );
        }
        if ( ! filter_var($_REQUEST['user']['email'], FILTER_VALIDATE_EMAIL)) {
            wp_send_json( array('error' => 'Invalid email') );
        }

        $args = array(
            'user' => $_REQUEST['user'],
            'diag' => array (
                'site_url' => trailingslashit( site_url() ),
                'php' => phpversion(),
                'web_server' => $_SERVER['SERVER_SOFTWARE'],
                'wp' => $wp_version,
                'plugin_version' => $this->plugin_data['Version'],
                'cachebust' => microtime()
            )
        );

        $args = $this->encode($args);
        $url = add_query_arg(
            'wpmr_action',
            'wpmr_register', 
            add_query_arg(
                'p',
                '495',
                add_query_arg('reg_details',$args, WPMR_SERVER)
                )
            );
    
        $response = wp_safe_remote_request(
            $url,
            array(
                'blocking' => true,
            )
        );
        if( is_wp_error($response) ) {
            wp_send_json( array('error' => $response->get_error_message()) );
        }
        
        $response = wp_remote_retrieve_body($response);
        
        if(empty($response) || is_null($response)){
            wp_send_json(array('error' => 'No response. Registration Failed.'));
        }

        $data = json_decode( $response, true );

        if(! isset($data['error'])) {
            delete_transient('WPMR_signatures');
            update_option( 'WPMR_reg', $data );
        }
        wp_send_json( $data );

    }

    function get_diag_data(){
        // diag data for server side validation, load balancing, spam protection
        global $wp_version;
        $current_user = wp_get_current_user();
        $data = array(
            'php' => phpversion(),
            'web_server' => $_SERVER['SERVER_SOFTWARE'],
            'wp' => $wp_version,
            'key' => md5(site_url()),
            'site_url' => trailingslashit( site_url() ),
            'signatures' => get_transient('WPMR_signatures')['v'] ,
            'plugin' => $this->plugin_data['Version'],
            'return_url' => esc_url( get_admin_url(null, 'options-general.php?page=wpmr') ),
            'origin_ajaxurl' => admin_url( 'admin-ajax.php' ),
            'origin_nonce' => wp_create_nonce( 'wpmr_gscapi' ),
            'user_firstname' => $current_user->user_firstname,
            'user_lastname' => $current_user->user_lastname,
            'user_email' => $current_user->user_email,
        );
        return $data;
    }

    function get_definitions_update_url(){
        $url = WPMR_SERVER;
        $args = array(
                'cachebust' => time(),
                'wpmr_action' => 'update-definitions',
            );
            $compatibility = $this->plugin_data;
            $state = get_option('WPMR_reg');
            if($state) {
                $state = array_merge($state, $compatibility);
            }
            else {
                $state = $compatibility;
            }
            $args['state'] = $this->encode($state);
        return trailingslashit($url) . '?' . urldecode(http_build_query($args));
    }

    function check_definition_updates(){

    }

    function update_definitions($force = false){ 
        if(! $force) {
            check_ajax_referer( 'wpmr_update_sigs', 'wpmr_update_nonce' );
        }
        $response = wp_safe_remote_request($this->get_definitions_update_url());
        $headers = wp_remote_retrieve_headers( $response );
        $status_code = wp_remote_retrieve_response_code( $response );
        if( 200 != $status_code ) {
            return;
        }
        if( is_wp_error($response) ) {
            return;
        }
        $body = wp_remote_retrieve_body( $response );
        $definitions = json_decode( $body, true );
        if($definitions['success'] != true) {
            return;
        }
        if(!empty($definitions['success']) && $definitions['success'] == true ){
            $definitions = $definitions['data'];
            set_transient('WPMR_signatures', $definitions, 1 * HOUR_IN_SECONDS);
            $time = date('U');
            update_option( 'WPMR_update_stats', array('sig_time' => $time ) );
            return true;
        }
    }

    function get_definitions(){
        $definitions = get_transient('WPMR_signatures');
        if( ! $definitions ) {
            if( $this->is_registered()){
                if($this->update_definitions(true)) {
                    $definitions = get_transient('WPMR_signatures');
                    unset($definitions['v']);
                    return $definitions;
                }
            }
            else {
                $definitions = file_get_contents( trailingslashit(__DIR__) . 'wpmr.json');
                $definitions = json_decode($definitions, true);
                set_transient('WPMR_signatures', $definitions, 1 * HOUR_IN_SECONDS);
                update_option( 'WPMR_update_stats', array('sig_time' => 0 ) );
                unset($definitions['v']);
                return $definitions;
            }
        }
        else {
            unset($definitions['v']);
            return $definitions;
        }
    }

    function add_admin_scripts(){
        wp_enqueue_script( 'jquery' );
        wp_enqueue_script( 'common' );
		wp_enqueue_script( 'wp-lists' );
		wp_enqueue_script( 'postbox' );
    }

    /* Add links below the plugin name on the plugins page */
    function plugin_action_links($links){
        $links[] = '<a href="'. esc_url( get_admin_url(null, 'options-general.php?page=wpmr') ) .'">Run Site Scan</a>';
        $links[] = '<a target="_blank" href="https://www.converticacommerce.com?item_name=Donation%20for%20WP%20Malware%20Removal&cmd=_donations&currency_code=USD&lc=US&business=shivanand@converticacommerce.com"><strong style="display:inline">Donate</strong></a>';
        return $links;
    }

    function plugin_meta_links($links, $file){
        if ($file !== plugin_basename(WPMR_PLUGIN)) {
            return $links;
        }
        $links[] = '<strong><a target="_blank" href="https://wp-malware-removal.com/support/" title="Malware Cleanup Service">Malware Support</a></strong>';
        $links[] = '<strong><a target="_blank" href="https://wp-malware-removal.com/" title="Website of this plugin">Plugin Homepage</a></strong>';
        $links[] = '<strong><a target="_blank" href="https://wordpress.org/plugins/wp-malware-removal/" title="Rate WP Malware Removal">Rate the plugin ★★★★★</a></strong>';
        $links[] = '<strong><a target="_blank" href="https://www.converticacommerce.com?item_name=Donation%20for%20WP%20Malware%20Removal&cmd=_xclick&currency_code=USD&business=shivanand@converticacommerce.com"><strong style="display:inline">Donate</strong></a></strong>';
        return $links;
    }

    function plugin_styles(){
        $screen = get_current_screen();
		if( $screen->id == 'toplevel_page_wpmr' || $screen->id == 'malware-removal_page_wpmr-tools' ) {
			wp_enqueue_style( 'wpmr-stylesheet', WPMR_PLUGIN_DIR_URL . 'assets/admin-styles.css' );
		}
    }

    function llog($str, $echo = true) {
        if($echo){
            echo '<pre>';
            print_r($str);
            echo '</pre>';
        }
        else {
            return print_r($str, 1);
        }
    }
    
    function settings_menu(){
        $hook_suffix = add_menu_page(
			'Malware Removal', 
			'Malware Removal', 
			'manage_options', 
			'wpmr', 
            array( $this, 'settings_page' ),
            $this->url . 'assets/icon.svg'
        );
        add_submenu_page( 'wpmr', 'Run Scan', 'Run Scan', 'manage_options' , 'wpmr', array($this, 'settings_page'));
        add_submenu_page( 'wpmr', 'Tools', 'Tools', 'manage_options' , 'wpmr-tools', array($this, 'tools_page'));
    }
    
    function tools_page(){
        $domain = untrailingslashit(preg_replace('/https?:\/\//','',get_site_url()));
        $domain = explode('/',$domain)[0];
        echo '
        <p><strong>Click on any of the links below to run your site through the tool.</strong> All links open in new tab / window.</p>
        <p><strong>Note:</strong>This tool is not affiliated with any of these sites.</p>
        <ol>
        <li><a id="tool" target="_blank" href="https://www.google.com/transparencyreport/safebrowsing/diagnostic/index.html#url='.get_site_url().'">Google Safe Browsing Diagnostic</a></li>
        <li><a id="tool" target="_blank" href="https://sitecheck.sucuri.net/results/'.get_site_url().'">Sucuri Website Malware Scanner</a></li>
        <li><a id="tool" target="_blank" href="https://www.virustotal.com/#/domain/'. $domain . '">VirusTotal Analyzer</a><br></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/screenshot/'.get_site_url().'">Geek Flare Screenshot Checker</a></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/blacklist-test/'.get_site_url().'">Geek Flare Blacklist Test</a></li>
        <li><a id="tool" target="_blank" href="https://geopeeker.com/fetch/?url='.get_site_url().'">GeoPeeker Peek Results</a><br></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/dnssec-test/'.get_site_url().'">Geek Flare DNSSEC Test</a></li>
        <li><a id="tool" target="_blank" href="https://tools.geekflare.com/report/https-test/'.get_site_url().'">Geek Flare HTTP/2 Test</a></li>
        </ol>
        ';
    }

    function get_last_updated_ago(){
        $updated = get_option( 'WPMR_update_stats' );
        if(! $updated) {
            return 'Never';
        }
        else {
            if(! empty($updated['sig_time'] ) ) {
                return human_time_diff( date('U', get_option( 'WPMR_update_stats' )['sig_time'] ), date('U') ) . ' ago';
            }
            else {
                return 'Never'; 
            }
        }
    }

    function settings_page(){
        $this->get_checksums();
        $this->get_definitions();
        ?>
        <div class="wrap">
            <h1>WP Malware Removal</h1>
            <div class="container-wrap">
                <div class="engine_stats">
                <span id="icon"></span>
                <table id="engine_stats">
                    <tr><th>Checksums:</th><td><span class="checksum_count"><?php echo count($this->get_checksums()); ?></span></td></tr>
                    <tr><th>Signatures:</th><td><span class="sig_count"> <?php  echo count(get_transient('WPMR_signatures')['definitions']) ; ?></span></td></tr>
                    <tr><th>Last Updated:</th><td><span class="sig_date"><?php echo $this->get_last_updated_ago(); ?></span></td></tr>
                    <tr><th>Total Files:</th><td><span class="total_files"><?php echo count($this->get_all_files()); ?></span></td></tr>
                    <tr><th>Files Excluded:</th><td><span class="files_excluded"><?php echo ( count($this->get_all_files()) - $this->bootstrap('bleh')['count'] ); ?></span></td></tr>
                    <tr><th>Files to scan:</th><td><span id="files_to_scan"><?php echo $this->bootstrap('bleh')['count']; ?></span></td></tr>
                    <tr><th>Files remaining:</th><td><span id="files_remaining"><?php echo $this->bootstrap('bleh')['count']; ?></span></td></tr>
                    <tr><th>Time Elapsed:</th><td><span id="time_elapsed">Not Initiated</span></td></tr>
                    <tr><th>Time Remaining:</th><td><span id="time_remaining">Not Initiated</span></td></tr>
                    <tr><th>Scan Speed:</th><td><span id="scan_speed">Duck</span></td></tr>
                    <tr><th>Status:</th><td><span class="engine_status">Ready</span></td></tr>
                    </table>
                </div>
                <div class="container">
                    <div class="gauge-a"></div>
                    <div class="gauge-c"></div>
                    <div class="gauge-data">
                        <h1 id="percent"></h1>
                    </div>
                </div>
                <div id="ux">
                    <div id="wpmr-fle-scroll-wrap"><span id="file-scroll"></span></div>
                    <div id="wpmr-scan-conrol-wrap"><input type="submit" value="Start Scan" id="scan-control" data-state="0" class="button-primary" /></div>
                    <div id="wpmr-god-wrap" title="Advanced options. Only check when you know what you are doing." ><input type="checkbox" id="wpmr-god" class="wpmr-god" /><label for="wpmr-god">GOD MODE</label></div>
                    <div id="wpmr-batchsize-wrap" title="Your webhost may block too many requests. Be careful." class="transparent"><input type="range" id="wpmr-batchsize" value="1" name="points" min="1" max="100" /></div>
                    <div id="wpmr-scan-instructions-wrap" title="Your webhost may block too many requests. Be careful."><span id="scan-instructions">1 File per req.</span></div>
                </div>
                <div id="lcd"></div>
            </div>
            
            <div id="poststuff">
                <div class="metabox-holder columns-2" id="post-body">
                    <div class="postbox-container" id="post-body-content">
                        <?php do_meta_boxes( 'toplevel_page_wpmr', 'main', null ); ?>
                    </div>
                    <!-- #postbox-container -->
                    <div id="postbox-container-1" class="postbox-container">
                    <?php
                        do_meta_boxes('toplevel_page_wpmr', 'side', null);
                    ?>
                    </div>
                </div>
            </div>
        </div>
        <script type="text/javascript">
        //<![CDATA[
        jQuery(document).ready( function ($) {
            $('#wpmr-god').change(function(){
                if ($(this).prop('checked')){
                    $('#wpmr-batchsize-wrap').removeClass('transparent');
                }
                else {
                    $('#wpmr-batchsize-wrap').addClass('transparent');
                }
            });
            $('#wpmr-batchsize').change(function(){
                val = $('#wpmr-batchsize').val();
                if(val > 1){
                    html = val + ' Files per req.';
                }
                else{
                    html = val + ' File per req.';
                }
                $('#scan-instructions').html( html ); 
            });
            // close postboxes that should be closed
            $('.if-js-closed').removeClass('if-js-closed').addClass('closed');
            // postboxes setup
            postboxes.add_postbox_toggles('toplevel_page_wpmr');
        });
        //]]>
        </script>
        <?php
    }

    function wpmr_inspect_file(){
        check_ajax_referer('wpmr_inspect_file', 'wpmr_inspect_nonce');
        wp_send_json($this->file_checks($_REQUEST['file']));
    }
 
    function file_checks($file){
        if(is_file($file) && is_readable($file) && (! filesize($file) <= 0) && filesize($file) < 10800000 ) { // file checks passed
            $content = file_get_contents( $file );
            if($content !== FALSE) { // file_get_contents succeeded
                $content = htmlspecialchars( $content, ENT_SUBSTITUTE );
                if(! $content) { // non-printable chars
                    wp_send_json(array('status' => 'error' , 'error' => 'File contains non-printable characters.'));
                } // non-printable chars
                wp_send_json(array('status' => 'success' , 'error' => '', 'content' => $content) );
            } // file_get_contents succeeded
            else { // file_get_contents failed
                wp_send_json(array('status' => 'error' , 'error' => 'Error getting file contents.'));
            } // file_get_contents failed
        } // file checks passed
        else { // file checks failed
            wp_send_json(array('status' => 'error' , 'error' => 'Empty or inaccessible or too large a file'));
        } // file checks failed
    }

    /* Ajax request handler for triggering scan */
    function wpmr_init_scan(){
        check_ajax_referer( 'wpmr_init_scan', 'wpmr_init_nonce' );
        wp_send_json($this->bootstrap($_REQUEST['timestamp']));
    }
    
    function bootstrap($timestamp) {
        //$this->reset_flog();
        $files = $this->get_all_files();
        if(is_array($files)) {
            $files = array_filter($files, function($value, $key) {
                return ! in_array($this->get_fileext($value), $this->get_excluded());
            }, ARRAY_FILTER_USE_BOTH);
        }
        return array('files' => $files, 'timestamp' => $timestamp, 'count' => count($files));
    }

    function wpmr_scan_files(){
        
        $files = $_REQUEST['files'];
        $tests = array();
        foreach($files as $file) {
            if( $this->fails_checksum($file) ) { // if file fails checksum
                $probable = $this->wpmr_scan_file_threats($file);
                if($probable) {
                    $tests[$file] = $probable;
                }
            }
        }
        wp_send_json(array(
            'report' => $tests,
            'memory'=> (memory_get_peak_usage(true)/1024/1024))
        );
    }

    function wpmr_scan_file_threats($file){
        if(@filesize($file) > 10800000 ) {
            return; // 
        }
        $ext = $this->get_fileext($file);
        if(in_array($ext, $this->get_excluded())) {
            return; // 'Excluded';
        }
        $tests = array();
        if(! is_readable($file)){
            return '<span class="threat skipped" title="WordPress Malware Removal could not access and read this file for scanning">Error Reading File</span>';
        }
        if(!is_file($file) || !($filesize = filesize($file)) || !($GLOBALS["WPMR"]["tmp"]["file_contents"] = @file_get_contents($file)) ) {
            return; // '<span class="skipped" title="">Something Fishy with file'.$file.': filesize:'.filesize($file).':</span>'; 
        }
        if( $GLOBALS["WPMR"]["tmp"]["file_contents"] === FALSE ) {
            return '<span class="threat skipped" title="WordPress Malware Removal could not access and read this file for scanning">Error Reading File</span>';
        }
        if( empty( $GLOBALS["WPMR"]["tmp"]["file_contents"] ) ) {
            return;// 'Empty' ; // why care?
        }
        $definitions = $this->get_definitions()['definitions'];
        foreach( $definitions as $definition => $signature ) {
            //$i++;    
            $matches = preg_match( $this->decode($signature['signature']), $GLOBALS["WPMR"]["tmp"]["file_contents"] );
            $pcre_err = preg_last_error();
            if($pcre_err != 0) {
                continue;
            }
            if(  $matches >= 1 ) {
                return '<span class="threat '.$signature['severity'].'">'.$signature['severity'].'</span>';
            }
        }
    }

    function get_excluded(){
        return apply_filters( 'wpmr_excluded_ext', array(
            '7z', 
            'bmp', 
            'bz2',
            'css', 
            'doc', 
            'docx', 
            'exe', 
            'fla', 
            'flv', 
            'gif', 
            'gz', 
            'jpg', 
            'jpeg', 
            'mo', 
            'mov', 
            'mp3', 
            'png',
            'psd', 
            'rar', 
            'svg', 
            'tif', 
            'tiff', 
            'ttf', 
            'pdf', 
            'po', 
            'pot', 
            'scss',
            'so', 
            'tar',
            'tgz',
            'z',
            'zip', 
        ));
    }

    function get_fileext($filename) {
        $nameparts = explode(".", ".$filename");
        return strtolower($nameparts[(count($nameparts)-1)]);
    }
        
    function reset_flog(){
        $date = date('Ymd-G:i:s'); // 20171231-23:59:59
        $file = $this->dir . 'log.log';
        file_put_contents($file, '', LOCK_EX);
    }

    function flog($str) {
        $date = date('Ymd-G:i:s'); // 20171231-23:59:59
        $date = $date . '-'.microtime(TRUE);
        $file = $this->dir . 'log.log';
        file_put_contents($file, PHP_EOL . $date, FILE_APPEND | LOCK_EX);
        usleep(1000);
        ob_flush();
        ob_start();
        print_r($str);
        file_put_contents($file, PHP_EOL . ob_get_flush(), FILE_APPEND | LOCK_EX);
        usleep(1000);
    }

    function fails_checksum($local_file) {
        $checksums = $this->get_checksums();
        $install_path = get_home_path();
        $file = array();
        $match_path = str_replace($install_path,'', $local_file );
        if(array_key_exists($match_path, $checksums)) {
            if(empty($checksums[$match_path])){
                return;
            }
            if(is_array($checksums[$match_path])) { // plugin readmes can have multiple md5 hashes since a readme.txt can be updated on svn without bumping the plugin version
                if(! in_array(md5_file($local_file), $checksums[$match_path] )) {
                    return true;
                }
                else{
                    return;
                }
            }
            else {
                if($checksums[$match_path] != md5_file($local_file)){
                    return true;
                }
                else{
                    return;
                }
            }
        }
        else{
            return true; // we found a file that's not a part of WordPress
        }
    }

    function get_all_files($path = false) {
        if(!$path) {
            $path = untrailingslashit( get_home_path() );
        }
        if(is_dir($path)) {
            $children = @scandir($path);
            if(is_array($children)) {
                $children = array_diff($children, array('..', '.'));
                $dirs = array();
                $files = array();
                foreach( $children  as $child ) {
                    $target = trailingslashit($path) . $child;
                    if(is_dir($target)) {
                        $elements = $this->get_all_files($target);
                        if($elements) { // check for read/write errors
                            foreach($elements as $element) {
                                $files[] = $element;
                            }
                        }
                    }
                    if(is_file($target)) {
                        $files[] = $target;
                    }
                }
                return $files;
            }
        }
    }

    function delete_checksums(){
        delete_transient('WPMR_checksums');
    }

    function get_checksums(){
        $checksums = get_transient( 'WPMR_checksums' );
        if(! $checksums) {
            global $wp_version;
            $checksums = get_core_checksums( $wp_version, get_locale() );
            $plugin_checksums = $this->get_plugin_checksums();
            if($plugin_checksums) {
                $checksums = array_merge($checksums, $plugin_checksums);
            }
            if($checksums) {
                set_transient( 'WPMR_checksums',$checksums, 12 * HOUR_IN_SECONDS );
                return $checksums;
            }
            return;
        }
        else {
            return $checksums;
        }
    }

    function get_theme_checksums(){
        
        $all_themes = wp_get_themes();
        $install_path = get_home_path();
        $this->llog($install_path);

        $theme_checksums = array();
        $theme_root = get_theme_root();
        foreach($all_themes as $key => $value) {
            $this->llog($key);
            
            $theme_file = trailingslashit($theme_root) . $key;
            $theme_file = str_replace($install_path,'', $theme_file );
            $this->llog( 'FILE:'. $theme_file);
            $checksum_url = 'https://downloads.wordpress.org/theme/'.$key.'/'.$value['Version'].'.json';
            $this->llog($checksum_url);
        }
        $this->llog($theme_checksums);
    }

    function get_plugin_checksums(){
        $all_plugins = get_plugins();
        $install_path = get_home_path();
        $plugin_checksums = array();
        foreach($all_plugins as $key => $value) {
            if ( false !== strpos( $key, '/' ) ) { // plugin has to be inside a directory. currently drop in plugins are not supported
                $plugin_file = trailingslashit(dirname($this->dir)) . $key;
                $plugin_file = str_replace($install_path,'', $plugin_file );
                $checksum_url = 'https://downloads.wordpress.org/plugin-checksums/'.dirname($key).'/'.$value['Version'].'.json';
                $checksum = wp_safe_remote_get($checksum_url);
                if(is_wp_error($checksum)) {
                    continue;
                }
                if( '200' != wp_remote_retrieve_response_code($checksum)) {
                    continue;
                }
                $checksum = wp_remote_retrieve_body($checksum);
                $checksum = json_decode($checksum, true);
                if(!empty($checksum['files'])){
                    $checksum = $checksum['files'];
                    foreach($checksum as $file => $checksums){
                        $plugin_checksums[trailingslashit(dirname($plugin_file)) . $file] = $checksums['md5'];
                    }
                }
            }
            else {
            }
        }
        return $plugin_checksums;
    }

    function read_error($path) {
        if (!file_exists($path)) {
            return 'are you sure this file exists?: '. $path;
        }
        if (!is_readable($path) ) {
            return 'are you sure this file is readable?:' . $path;
        }
        return (is_array($error) && isset($error["message"])?preg_replace('/[\r\n]/', ' ', print_r($error["message"],1)):"readable?");
    }

    function plugin_update_message($data, $response){
        $changelog = 'https://plugins.trac.wordpress.org/browser/'.basename($this->dir).'/trunk/readme.txt?format=txt&cachebust='.time(); // should translate into https://plugins.trac.wordpress.org/browser/wp-real-time-social-proof/trunk/readme.txt?format=txt since repo doesn't allow changing slugs
        $res = wp_safe_remote_get($changelog);
        if(is_wp_error($res)) {
            return;
        }
        $res = wp_remote_retrieve_body($res);
        $regexp = '~==\s*Changelog\s*==\s*=\s*[0-9.]+\s*=(.*)(=\s*' . preg_quote( $this->plugin_data['Version'] ) . '\s*=|$)~Uis';//$res  = stristr( $res, '== Changelog ==', true );
        if ( !preg_match( $regexp, $res, $matches ) ) {
            return;
        }
        $changelog = (array) preg_split( '~[\r\n]+~', trim( $matches[1] ) );
        //$ul = false;
        $upgrade_notice = '';
        foreach ( $changelog as $index => $line ) {
            if ( preg_match( '~^\s*\*\s*~', $line ) ) {
                $line = preg_replace( '~^\s*\*\s*~', '', htmlspecialchars( $line ) );
                $upgrade_notice .=  '<span style="font-weight:bold;">&#x2605;</span> '.$line . '<br />';
            } else {
            }
        }
        $upgrade_notice =  '<strong>Upgrading is a must to ensure that this plugin works with the latest signatures.</strong><br />'.$upgrade_notice;
        echo '<br /><br /><span style="display:block; border: 1px solid hsl(200, 100%, 80%); padding: 1em; background: hsl(200, 100%, 90%); line-height:2">'.$upgrade_notice.'</span>';
    }

    /* JS routine to handle user-events and reporting */
    function scripts(){
        ?>
        <script type="text/javascript">
        jQuery(document).ready(function ($) { //wrapper
            //$('#wpmr-inspect-box').hide();
            $('#wpmr-results-box').addClass('closed');
            $('#wpmr-inspect-box').addClass('closed');
            //$('#wpmr-support-box').addClass('closed');
            $("#wpmr-update").click(function (e) { //event
                e.preventDefault();
                wpmr_update = {
                    wpmr_update_nonce: '<?php echo wp_create_nonce( 'wpmr_update_sigs' ); ?>',
                    action: "wpmr_update_sigs",
                    cachebust: Date.now(),
                };
                $.ajax({
                    url: ajaxurl,
                    method: 'POST',
                    data: wpmr_update,
                    //timeout: 1,
                    complete: function(jqXHR, textStatus){ 
                        console.dir(jqXHR);
                        if(textStatus == 'success' || textStatus == 'notmodified') {
                            last_updated = new Date();
                            $('#last_updated').html( last_updated );
                        }
                    },
                    success: function (response) {
                        
                    } // initialize
                }); // ajax post
            });
            $("#scan-control").click(function (e) { //event
                e.preventDefault();
                record = new Date().getTime();
                $('.container .gauge-c').css('transform', 'rotate( 0turn)');
                $('#scan-results').html('');
                $('#percent').html('');
                $('#lcd').html('');
                $('.engine_status').toggleClass('blink');
                $('#icon').toggleClass('running');
                $('.engine_status').html('initiating…');
                total_files = 0;
                $("#scan-control").attr('disabled', 'disabled');
                $("#wpmr-batchsize").attr('disabled', 'disabled');
                $("#wpmr-god").attr('disabled', 'disabled');
                wpmr_init_scan = {
                    wpmr_init_nonce: '<?php echo wp_create_nonce( 'wpmr_init_scan' ); ?>',
                    action: "wpmr_init_scan",
                    timestamp: record,
                    cachebust: Date.now(),
                };
                $.ajax({
                    url: ajaxurl,
                    method: 'POST',
                    data: wpmr_init_scan,
                    complete: function(jqXHR, textStatus){
                        },
                    success: function (response) {
                        files = response.files;
                        files = Object.values(files);
                        $('.engine_status').html('initialized…');
                        last = 0;
                        results = [];
                        old_time = Date.now();
                        results.starttime = old_time;
                        results.totalfiles = files.length;
                        results.memory = 0;
                        results.infectedfiles = {};
                        found = 0;
                        if(files.length){
                            $('.engine_status').html('scanning…');
                            sent = Date.now();
                            scan_files();
                            $('#wpmr-results-box').removeClass('closed');
                        }
                        function scan_files(){
                            batchsize = $('#wpmr-batchsize').val();
                            $('#wpmr-batchsize').attr('disabled', 'disabled');
                            
                            if(!files.length){
                                return;
                            }
                            wpmr_scan_files = {
                                action: "wpmr_scan_files",
                                files:  ( files.length ) ? ( files.length > batchsize ? files.splice(0, batchsize)  : files ) : false 
                            };
                            lastbatch = wpmr_scan_files.files;
                            siteroot = '<?php echo get_home_path(); ?>';
                            $('#file-scroll').html('../' + lastbatch[0].replace(siteroot,''));
                            $.ajax({
                                url: ajaxurl,
                                method: 'POST',
                                data: wpmr_scan_files,
                                success: function (scan_status) {
                                },
                                complete(jqXHR,  textStatus){
                                    //console.log('received in ' + (Date.now() - sent)) ;
                                    var scan_status = {};
                                    if(textStatus == "success"){
                                        scan_status = jqXHR.responseJSON.report;
                                    }
                                    else {  // ajax failure
                                        Object.entries(lastbatch).forEach(([key, value]) => {
                                            scan_status[value] = '<span class="threat skipped" title="Server returned invalid information while scanning this file. Don\'t worry. You can always rescan.">Invalid Scan Response</span>';
                                        });
                                    }
                                    new_time = Date.now();
                                    old_time = Date.now();
                                    html = ''; 
                                    if(Object.keys(scan_status).length){
                                        Object.entries(scan_status).forEach(([key, value]) => {
                                            results.infectedfiles[key] = value;
                                        }); 
                                        found = Object.keys(results.infectedfiles).length;
                                    }
                                    if(found){
                                        html = '<table id="scan-records"><thead><tr><th>Status</th><th>File</th><th></th></tr></thead><tbody>';
                                        $('#wpmr-what-to-do').show();
                                    }
                                    Object.entries(results.infectedfiles).forEach(([key, value]) => {
                                        html += '<tr class="dected"><td class="level">' + value + '</td><td class="inspect"><span class="inspect_file button button-secondary button-small" data-file="' +key+ '">Inspect</span></td><td class="infected-file"><pre class="recorded-file">' + key + '</pre></td></tr>';
                                    });
                                    if(found){
                                        html += '</tbody></table>';
                                    }
                                    else {
                                        html = '<h1 class="blink">All Well. Nothing yet&hellip; 🙂</h1>' ;
                                    }
                                    $('#scan-results').html( html );
                                    results.filesremaining = files.length
                                    if( textStatus == "success" && jqXHR.hasOwnProperty('responseJSON') && jqXHR.responseJSON.hasOwnProperty('memory') && jqXHR.responseJSON.memory > results.memory ) {
                                        results.memory = jqXHR.responseJSON.memory;
                                    }
                                    results.scanspeed =  ( results.totalfiles - results.filesremaining ) / ((Date.now() - results.starttime) / 1000 )  ;
                                    $('#lcd').html('<span class="stats"><span class="speed"><span class="digits">' + (results.scanspeed).toFixed(0) + '</span><span class="rate">&nbsp;files / sec</span></span><span class="ram">' + parseFloat( results.memory ).toFixed(0) + 'mb ram</span>').fadeIn(2000);
                                    $('#files_remaining').html(results.filesremaining);
                                    results.timeelapsed =  (Date.now() - results.starttime) ;
                                    $('#time_elapsed').html( msToTime(results.timeelapsed)  );
                                    $('#scan_speed').html((results.scanspeed).toFixed(0) + ' files / sec.');
                                    results.timeremaining = results.filesremaining / results.scanspeed;
                                    $('#time_remaining').html( msToTime( results.timeremaining * 1000 ) );
                                    percent = ( ( results.totalfiles - results.filesremaining )  / results.totalfiles ) * 100 ;
                                    if(last) {
                                        $('#files_remaining').html('0');
                                        $('#icon').toggleClass('running');
                                        $('.engine_status').toggleClass('blink');
                                        $('.engine_status').html('complete');
                                        $('#time_remaining').html('finished');
                                        $('#scan_speed').html('done');
                                        $('#files_to_scan').html('none');
                                        $('#file-scroll').html('');
                                        percent = 100;
                                        $('html,body').animate({scrollTop: $('#scan-results').offset().top},'slow');
                                        if(found) {
                                            $('#wpmr-support-box').removeClass('closed');
                                        }
                                        else {
                                            $('#scan-results').html( '<h1>All Well. Nothing to do 🙂</h1>' );
                                        }
                                        $("#scan-control").val('Re-Scan');
                                        $("#scan-control").removeAttr('disabled');
                                        $('#wpmr-batchsize').removeAttr('disabled');
                                        $("#wpmr-god").removeAttr('disabled');
                                    }
                                    $('#percent').html('<span class="percentage">' + parseFloat( percent ).toFixed(0) + '%</span><br /><span class="time">' + msToTime(results.timeelapsed) + '</span>');
                                    $('.container .gauge-c').css('transform', 'rotate( ' + ( .005 * percent ) + 'turn)');
                                    if(last != 1) {
                                        if(files.length <= batchsize ) {
                                            last = 1;
                                        }
                                        sent = Date.now();
                                        scan_files();
                                    }
                                },
                                error: function( jqXHR,  textStatus,  errorThrown){} 
                            });
                        }
                    }, // initialize
                }); // ajax post
            }); //click
            $('#scan-results').on('click', '.inspect_file', function(event) {
            event.preventDefault();
            file = $( this ).attr('data-file');
            if(file){
                wpmr_inspect_file = {
                    wpmr_inspect_nonce: '<?php echo wp_create_nonce( 'wpmr_inspect_file' ); ?>',
                    action: "wpmr_inspect_file",
                    file: file
                };    
                $.ajax({
                    url: ajaxurl,
                    method: 'GET',
                    data:  wpmr_inspect_file,
                    complete: function(jqXHR, textStatus){
                        $('#wpmr-inspect-box').removeClass('closed');
                        console.log('init wpmr_inspect_file'); 
                        console.dir(jqXHR); 
                        console.dir(textStatus);
                        if(textStatus == 'success') {
                            if(jqXHR.hasOwnProperty('responseJSON') ) {
                                if(jqXHR.responseJSON.hasOwnProperty('content') && jqXHR.responseJSON.content.length) {
                                    $('#inspect_file_path').html('<strong>File:</strong> <code>'+file+'</code>'); 
                                    $('#inspect_file').html(jqXHR.responseJSON.content);
                                }
                                else {
                                    if(jqXHR.responseJSON.hasOwnProperty('error')) {
                                        $('#inspect_file').html(jqXHR.responseJSON.error);
                                    }
                                }
                            }
                            else {
                                $('#inspect_file').html('Invalid response from server');
                            }
                        } 
                        else {
                            $('#inspect_file').html('Request failed.');
                        }
                        $('html,body').animate({scrollTop: $('#inspect_file').offset().top},'slow');
                    },
                });
            }
            });
        }); //ready
        function msToTime(duration) {
            var milliseconds = parseInt((duration % 1000) / 100),
            seconds = Math.floor((duration / 1000) % 60),
            minutes = Math.floor((duration / (1000 * 60)) % 60),
            hours = Math.floor((duration / (1000 * 60 * 60)) % 24);
            hours = (hours < 10) ? "0" + hours : hours;
            minutes = (minutes < 10) ? "0" + minutes : minutes;
            seconds = (seconds < 10) ? "0" + seconds : seconds;
            if(parseInt(hours)){
                return hours + ":" + minutes + ":" + seconds;
            }
            else{
                return minutes + ":" + seconds;
            }
            return hours + ":" + minutes + ":" + seconds + "." + milliseconds;
        }

        function millisToMinutesAndSeconds(millis) {
            var minutes = Math.floor(millis / 60000);
            var seconds = ((millis % 60000) / 1000).toFixed(0);
            return minutes + ":" + (seconds < 10 ? '0' : '') + seconds;
        }
        </script>
        <?php
    }

    function is_registered() {
        return get_option('WPMR_reg');
    }

    function encode($str) {
        return strtr( base64_encode( json_encode($str) ), '+/=', '-_,' );
    }

    function decode($str){
        return json_decode( base64_decode( strtr( $str, '-_,', '+/=' ) ), true );
    }

}

function wp_malware_removal(){
    return WPMR_Init::get_instance();
}

wp_malware_removal();
